@@ -20,9 +20,7 @@
 #
 
 import logging
-from typing import TYPE_CHECKING, Dict, Optional, cast
-
-from typing_extensions import Literal
+from typing import TYPE_CHECKING, Dict, Literal, Optional, cast
 
 from synapse.api.errors import (
     Codes,
