@@ -28,6 +28,7 @@ from http import HTTPStatus
 from typing import (
     TYPE_CHECKING,
     List,
+    Literal,
     Mapping,
     Optional,
     Sequence,
@@ -37,8 +38,6 @@ from typing import (
     overload,
 )
 
-from typing_extensions import Literal
-
 from twisted.web.server import Request
 
 from synapse._pydantic_compat import (
