@@ -2,7 +2,7 @@
 # This file is licensed under the Affero General Public License (AGPL) version 3.
 #
 # Copyright 2022 The Matrix.org Foundation C.I.C.
-# Copyright (C) 2023 New Vector, Ltd
+# Copyright (C) 2023-2024 New Vector, Ltd
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -34,38 +34,19 @@ if TYPE_CHECKING:
 logger = logging.getLogger(__name__)
 
 
-class RendezvousServlet(RestServlet):
-    """
-    This is a placeholder implementation of [MSC3886](https://github.com/matrix-org/matrix-spec-proposals/pull/3886)
-    simple client rendezvous capability that is used by the "Sign in with QR" functionality.
-
-    This implementation only serves as a 307 redirect to a configured server rather than being a full implementation.
-
-    A module that implements the full functionality is available at: https://pypi.org/project/matrix-http-rendezvous-synapse/.
-
-    Request:
-
-    POST /rendezvous HTTP/1.1
-    Content-Type: ...
-
-    ...
-
-    Response:
-
-    HTTP/1.1 307
-    Location: <configured endpoint>
-    """
-
+class MSC4108DelegationRendezvousServlet(RestServlet):
     PATTERNS = client_patterns(
-        "/org.matrix.msc3886/rendezvous$", releases=[], v1=False, unstable=True
+        "/org.matrix.msc4108/rendezvous$", releases=[], v1=False, unstable=True
     )
 
     def __init__(self, hs: "HomeServer"):
         super().__init__()
-        redirection_target: Optional[str] = hs.config.experimental.msc3886_endpoint
+        redirection_target: Optional[str] = (
+            hs.config.experimental.msc4108_delegation_endpoint
+        )
         assert (
             redirection_target is not None
-        ), "Servlet is only registered if there is a redirection target"
+        ), "Servlet is only registered if there is a delegation target"
         self.endpoint = redirection_target.encode("utf-8")
 
     async def on_POST(self, request: SynapseRequest) -> None:
@@ -73,9 +54,23 @@ class RendezvousServlet(RestServlet):
             request, self.endpoint, statusCode=TEMPORARY_REDIRECT, cors=True
         )
 
-    # PUT, GET and DELETE are not implemented as they should be fulfilled by the redirect target.
+
+class MSC4108RendezvousServlet(RestServlet):
+    PATTERNS = client_patterns(
+        "/org.matrix.msc4108/rendezvous$", releases=[], v1=False, unstable=True
+    )
+
+    def __init__(self, hs: "HomeServer") -> None:
+        super().__init__()
+        self._handler = hs.get_rendezvous_handler()
+
+    def on_POST(self, request: SynapseRequest) -> None:
+        self._handler.handle_post(request)
 
 
 def register_servlets(hs: "HomeServer", http_server: HttpServer) -> None:
-    if hs.config.experimental.msc3886_endpoint is not None:
-        RendezvousServlet(hs).register(http_server)
+    if hs.config.experimental.msc4108_enabled:
+        MSC4108RendezvousServlet(hs).register(http_server)
+
+    if hs.config.experimental.msc4108_delegation_endpoint is not None:
+        MSC4108DelegationRendezvousServlet(hs).register(http_server)
