@@ -34,6 +34,7 @@ from typing import (
     Generic,
     Iterable,
     List,
+    Literal,
     Optional,
     Set,
     Tuple,
@@ -44,8 +45,6 @@ from typing import (
     overload,
 )
 
-from typing_extensions import Literal
-
 from twisted.internet import reactor
 from twisted.internet.interfaces import IReactorTime
 
